import { initDb } from '.';
import uri from '../lib/ids';
import { getFamily } from './lib';
import { putNewRte } from './rte';
import { putNewRtept } from './rtept';
import { putNewTrk } from './trk';
import { putNewTrkpt } from './trkpt';
import { putNewTrkseg } from './trkseg';
import { putNewWpt } from './wpt';

declare global {
  var db: any;
}

const originalDateNow = globalThis.Date.now;

describe('getFamily', () => {
  beforeEach(async () => {
    await initDb({});
    globalThis.Date.now = () => 0;
  });
  afterEach(async () => {
    await db.destroy();
    db = undefined;
    globalThis.Date.now = originalDateNow;
  });
  test('returns two rows after a gpx and a track have been inserted.', async () => {
    await putNewTrk();
    const allDocs: any = await getFamily('gpx/');
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/4320000000000000",
      "key": "gpx/4320000000000000",
      "value": Object {
        "rev": "1-49baa096ec0c89962f2cafd3ff50b80b",
      },
    },
    Object {
      "id": "gpx/4320000000000000/2trk/000000",
      "key": "gpx/4320000000000000/2trk/000000",
      "value": Object {
        "rev": "1-46d2f0dcad2299e6ba2830cb1e10b234",
      },
    },
  ],
  "total_rows": 5,
}
`);
  });
  test('also returns the docs if required.', async () => {
    await putNewTrk();
    const allDocs: any = await getFamily('gpx/', {
      include_docs: true,
    });
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "doc": Object {
        "_id": "gpx/4320000000000000",
        "_rev": "1-49baa096ec0c89962f2cafd3ff50b80b",
        "doc": Object {
          "$": Object {
            "creator": "dyomedea version 0.000002",
            "version": "1.1",
            "xmlns": "http://www.topografix.com/GPX/1/1",
            "xmlns:dyo": "http://xmlns.dyomedea.com/",
            "xmlns:gpxtpx": "http://www.garmin.com/xmlschemas/TrackPointExtension/v1",
            "xmlns:gpxx": "http://www.garmin.com/xmlschemas/GpxExtensions/v3",
            "xmlns:wptx1": "http://www.garmin.com/xmlschemas/WaypointExtension/v1",
            "xmlns:xsi": "http://www.w3.org/2001/XMLSchema-instance",
            "xsi:schemaLocation": "http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd http://www.garmin.com/xmlschemas/GpxExtensions/v3 http://www8.garmin.com/xmlschemas/GpxExtensionsv3.xsd http://www.garmin.com/xmlschemas/WaypointExtension/v1 http://www8.garmin.com/xmlschemas/WaypointExtensionv1.xsd http://www.garmin.com/xmlschemas/TrackPointExtension/v1 http://www.garmin.com/xmlschemas/TrackPointExtensionv1.xsd",
          },
          "metadata": Object {
            "time": "1970-01-01T00:00:00.000Z",
          },
        },
        "type": "gpx",
      },
      "id": "gpx/4320000000000000",
      "key": "gpx/4320000000000000",
      "value": Object {
        "rev": "1-49baa096ec0c89962f2cafd3ff50b80b",
      },
    },
    Object {
      "doc": Object {
        "_id": "gpx/4320000000000000/2trk/000000",
        "_rev": "1-46d2f0dcad2299e6ba2830cb1e10b234",
        "doc": Object {
          "number": 0,
        },
        "type": "trk",
      },
      "id": "gpx/4320000000000000/2trk/000000",
      "key": "gpx/4320000000000000/2trk/000000",
      "value": Object {
        "rev": "1-46d2f0dcad2299e6ba2830cb1e10b234",
      },
    },
  ],
  "total_rows": 5,
}
`);
  });
  test('returns three rows after a gpx and a track segment have been inserted.', async () => {
    await putNewTrkseg();
    const allDocs: any = await getFamily('gpx/');
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/4320000000000000",
      "key": "gpx/4320000000000000",
      "value": Object {
        "rev": "1-49baa096ec0c89962f2cafd3ff50b80b",
      },
    },
    Object {
      "id": "gpx/4320000000000000/2trk/000000",
      "key": "gpx/4320000000000000/2trk/000000",
      "value": Object {
        "rev": "1-46d2f0dcad2299e6ba2830cb1e10b234",
      },
    },
    Object {
      "id": "gpx/4320000000000000/2trk/000000/000000",
      "key": "gpx/4320000000000000/2trk/000000/000000",
      "value": Object {
        "rev": "1-9ecf6e716dbbb26576a0b13a062b8f5f",
      },
    },
  ],
  "total_rows": 6,
}
`);
  });
  test('returns four rows after a gpx and a track point have been inserted.', async () => {
    await putNewTrkpt();
    const allDocs: any = await getFamily('gpx/');
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/4320000000000000",
      "key": "gpx/4320000000000000",
      "value": Object {
        "rev": "1-49baa096ec0c89962f2cafd3ff50b80b",
      },
    },
    Object {
      "id": "gpx/4320000000000000/2trk/000000",
      "key": "gpx/4320000000000000/2trk/000000",
      "value": Object {
        "rev": "1-46d2f0dcad2299e6ba2830cb1e10b234",
      },
    },
    Object {
      "id": "gpx/4320000000000000/2trk/000000/000000",
      "key": "gpx/4320000000000000/2trk/000000/000000",
      "value": Object {
        "rev": "1-9ecf6e716dbbb26576a0b13a062b8f5f",
      },
    },
    Object {
      "id": "gpx/4320000000000000/2trk/000000/000000/000000",
      "key": "gpx/4320000000000000/2trk/000000/000000/000000",
      "value": Object {
        "rev": "1-99c4c054903f577d66ddac0384e04d06",
      },
    },
  ],
  "total_rows": 7,
}
`);
  });

  test('returns two rows after a gpx and a waypoint have been inserted.', async () => {
    await putNewWpt();
    const allDocs: any = await getFamily('gpx/');
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/4320000000000000",
      "key": "gpx/4320000000000000",
      "value": Object {
        "rev": "1-49baa096ec0c89962f2cafd3ff50b80b",
      },
    },
    Object {
      "id": "gpx/4320000000000000/0wpt/000000",
      "key": "gpx/4320000000000000/0wpt/000000",
      "value": Object {
        "rev": "1-a3213e8257fb142c09f8b7d7c9482dac",
      },
    },
  ],
  "total_rows": 5,
}
`);
  });

  test('returns two rows after a gpx and a route have been inserted.', async () => {
    await putNewRte();
    const allDocs: any = await getFamily('gpx/');
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/4320000000000000",
      "key": "gpx/4320000000000000",
      "value": Object {
        "rev": "1-49baa096ec0c89962f2cafd3ff50b80b",
      },
    },
    Object {
      "id": "gpx/4320000000000000/1rte/000000",
      "key": "gpx/4320000000000000/1rte/000000",
      "value": Object {
        "rev": "1-a683896687f08ed73ceede02dc0c210e",
      },
    },
  ],
  "total_rows": 5,
}
`);
  });

  test('returns three rows after a gpx and a route point have been inserted.', async () => {
    await putNewRtept();
    const allDocs: any = await getFamily('gpx/');
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/4320000000000000",
      "key": "gpx/4320000000000000",
      "value": Object {
        "rev": "1-49baa096ec0c89962f2cafd3ff50b80b",
      },
    },
    Object {
      "id": "gpx/4320000000000000/1rte/000000",
      "key": "gpx/4320000000000000/1rte/000000",
      "value": Object {
        "rev": "1-a683896687f08ed73ceede02dc0c210e",
      },
    },
    Object {
      "id": "gpx/4320000000000000/1rte/000000/000000",
      "key": "gpx/4320000000000000/1rte/000000/000000",
      "value": Object {
        "rev": "1-e2eff2ff270417d00f681fcc3f7ceafe",
      },
    },
  ],
  "total_rows": 6,
}
`);
  });
});
